@@ -15,9 +15,9 @@ from datetime import datetime
 from email.mime.application import MIMEApplication
 from email.mime.multipart import MIMEMultipart
 from email.mime.text import MIMEText
-
+from wpscan_out_parse.formatter import format_results
 from wpwatcher import log, VERSION
-from wpwatcher.utils import get_valid_filename
+from wpwatcher.utils import get_valid_filename, replace
 
 # Date format used everywhere
 DATE_FORMAT='%Y-%m-%dT%H-%M-%S'
@@ -26,6 +26,7 @@ DATE_FORMAT='%Y-%m-%dT%H-%M-%S'
 mail_lock = threading.Lock()
 
 class WPWatcherNotification():
+    '''Send conditions logic + build and send mail reports'''
     
     def __init__(self, conf):
         # store specific mailserver values
@@ -49,17 +50,18 @@ class WPWatcherNotification():
         # mail server, will be created when sending mails
         self.server=None
 
+        self.use_monospace_font=conf['use_monospace_font']
+
     def notify(self, wp_site, wp_report, last_wp_report):
-        # Will print parsed readable Alerts, Warnings, etc as they will appear in email reports
-        log.debug("%s\n"%(WPWatcherNotification.build_message(wp_report, 
-                warnings=self.send_warnings or self.send_infos, # switches to include or not warnings and infos
-                infos=self.send_infos)))
+        '''Notify recipients if match conditions
+        '''
         if self.should_notify(wp_report, last_wp_report):
             self._notify(wp_site, wp_report, last_wp_report)
             return True
         else: return False
 
     def send_mail(self, message, to):
+        '''Raw sendmail'''
         # Connecting and sending
         self.server = smtplib.SMTP(self.smtp_server)
         self.server.ehlo_or_helo_if_needed()
@@ -72,7 +74,8 @@ class WPWatcherNotification():
         self.server.quit()
 
     # Send email report with status and timestamp
-    def send_report(self, wp_report, email_to) : #, send_infos=False, send_warnings=True, send_errors=False, attach_wpscan_output=False):
+    def send_report(self, wp_report, email_to) :
+        '''Build MIME message based on report and call send_mail'''
 
         # Building message
         message = MIMEMultipart("html")
@@ -81,11 +84,11 @@ class WPWatcherNotification():
         message['To'] = email_to
 
         # Email body
-        body=self.build_message(wp_report, 
-            warnings=self.send_warnings or self.send_infos, # switches to include or not warnings and infos
-            infos=self.send_infos )
+        body=self.build_message(wp_report, warnings=self.send_warnings, infos=self.send_infos)
+        if self.use_monospace_font:    
+            body = '<font face="Courier New, Courier, monospace" size="-1">'+body+'</font>'
 
-        message.attach(MIMEText(body))
+        message.attach(MIMEText(body, 'html' ))
         
         # Attachment log if attach_wpscan_output
         if self.attach_wpscan_output:
@@ -113,6 +116,7 @@ class WPWatcherNotification():
         log.info("Email sent: %s to %s" % (message['Subject'], email_to))
 
     def should_notify(self, wp_report, last_wp_report):
+        '''Determine if the notification should be sent'''
         should=True
         if not wp_report: return False
         
@@ -146,7 +150,7 @@ class WPWatcherNotification():
         return should
 
     def _notify(self, wp_site, wp_report, last_wp_report):
-
+        '''Sending the report'''
         # Send the report to
         if len(self.email_errors_to)>0 and wp_report['status']=='ERROR':
             to = ','.join( self.email_errors_to )
@@ -166,39 +170,23 @@ class WPWatcherNotification():
                 return True
                 
         # Handle send mail error
-        except (smtplib.SMTPException, ConnectionRefusedError, TimeoutError):
+        except (smtplib.SMTPException, ConnectionRefusedError, TimeoutError) as err:
             log.error("Unable to send mail report for site " + wp_site['url'] + "\n" + traceback.format_exc())
             wp_report['errors'].append("Unable to send mail report" + "\n" + traceback.format_exc())
-            raise RuntimeError("Unable to send mail report")
-            # Fail fast
-            #  if not self.check_fail_fast(): return False 
+            raise RuntimeError("Unable to send mail report") from err
 
     @staticmethod
     def build_message(wp_report, warnings=True, infos=False):
+        '''Build mail message text base on report and warnngs and info switch'''
         
-        message="WordPress security scan report for site: %s\n" % (wp_report['site'])
-        message+="Scan datetime: %s\n" % (wp_report['datetime'])
+        message="<p>WordPress security scan report for site: %s<br />\n" % (wp_report['site'])
+        message+="Scan datetime: %s<br />\n<p>" % (wp_report['datetime'])
         
-        if wp_report['errors'] : message += "\nAn error occurred."
-        elif wp_report['alerts'] : message += "\nVulnerabilities have been detected by WPScan."
-        elif wp_report['warnings']: message += "\nIssues have been detected by WPScan."
-        if wp_report['fixed']: message += "\nSome issues have been fixed since last scan."
-
-        message += WPWatcherNotification.format_issues('Errors',wp_report['errors'])
-        message += WPWatcherNotification.format_issues('Alerts',wp_report['alerts'])
-        message += WPWatcherNotification.format_issues('Fixed',wp_report['fixed'])
-        if warnings: message += WPWatcherNotification.format_issues('Warnings',wp_report['warnings'])
-        if infos: message += WPWatcherNotification.format_issues('Informations',wp_report['infos'])
-                
-        message += "\n\n--"
-        message += "\nWPWatcher -  Automating WPscan to scan and report vulnerable Wordpress sites"
-        message += "\nServer: %s - Version: %s\n"%(socket.gethostname(),VERSION)
-        return message
-
-    
-    @staticmethod
-    def format_issues(title, issues):
-        message=""
-        if issues:
-            message += "\n\n\t%s\n\t%s\n\n"%(title, '-'*len(title))+"\n\n".join(issues)
+        # Adjust wpscan_out_parse 'error' key in results 
+        wp_report['error']='\n'.join(wp_report['errors'])
+        message+=format_results(wp_report, format='html')
+            
+        message += "<br />\n<br />\n--"
+        message += "<br />\nWPWatcher -  Automating WPscan to scan and report vulnerable Wordpress sites"
+        message += "<br />\nServer: %s - Version: %s<br />\n"%(socket.gethostname(),VERSION)
         return message
\ No newline at end of file
